/*****************************************************************************
* Copyright (c) Bridgetek Pte Ltd.
* Software License Agreement
*
* This code is provided as an example only and is not guaranteed by Bridgetek. 
* BRIDGETEK accept no responsibility for any issues resulting from its use. 
* The developer of the final application incorporating any parts of this 
* sample project is responsible for ensuring its safe and correct operation 
* and for any consequences resulting from its use.
*****************************************************************************/
/**
* @file  App_RotaryDial
* @brief Sample application to demonstrate EVE primitives and widgets.
Versoin 5.0 - April/17/2017 - Restructure source code
*/

#include "Platform.h"
#include "App_RotaryDial.h"
#include "App_Common.h"



#define DIAL		1


/* Global used for context hal */
Gpu_Hal_Context_t host,*phost;

int32_t BaseTrackVal = 0,BaseTrackValSign = 0,MemLoc=0,BaseTrackValInit = 0;

uint32_t Load_RawDataFromfile(uchar8_t ImageArrayname[],/* Image Array*/
								 uint32_t ptr,/* Array Size*/
								 uint32_t RamAddr,
								 int16_t Handle,
								 int16_t Format,
								 int16_t Stride,
								 int16_t Width,
								 int16_t Height,
								 uint8_t i
								 )
{
	
	int32_t fsize=0;
	fsize = Uncompressed_FileSize[i];

        RamAddr = (uint32_t)(RamAddr + 7) & (~7);
	Gpu_Hal_WrCmd32(phost,  CMD_INFLATE);
	Gpu_Hal_WrCmd32(phost,  RamAddr);

#ifdef MSVC_PLATFORM 
        Gpu_Hal_WrCmdBuf(phost,ImageArrayname,ptr);
#endif
#ifdef ARDUINO_PLATFORM
	Gpu_Hal_WrCmdBufFromFlash(phost,ImageArrayname,ptr);
#endif

	App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(Handle));
	App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(RamAddr));
	App_WrCoCmd_Buffer(phost, BITMAP_LAYOUT(Format,Stride,Height));		
	App_WrCoCmd_Buffer(phost, BITMAP_SIZE(NEAREST,BORDER,BORDER,Width,Height));		
	App_WrCoCmd_Buffer(phost, VERTEX2II(0,0,Handle,0));				
		

	fsize = (fsize + 7) & (~7);//align the buffer to 8
	return (RamAddr + fsize);
}


/* Definitions used for optimizations */
#define ROTARY_APP_WHITECIRCELE_FIFO_SIZE (4*6+4*7*10+4*1)
#define ROTARY_APP_START_FIFO_SIZE_INWORDS (59)
#define ROTARY_APP_START_FIFO_SIZE (4*ROTARY_APP_START_FIFO_SIZE_INWORDS)
#define ROTARY_APP_START_FIFO_RANDOMBITMAP_SIZE (4*4)
#define ROTARY_APP_END_FIFO_SIZE_INWORDS (48)
#define ROTARY_APP_END_FIFO_SIZE (4*ROTARY_APP_END_FIFO_SIZE_INWORDS)
#define ROTARY_APP_BLOCK1_SIZE_INWORDS (17)
#define ROTARY_APP_BLOCK1_SIZE (4*ROTARY_APP_BLOCK1_SIZE_INWORDS)

/* Array optimization for code space */
#if 1        
       PROGMEM prog_uint32_t ROTARY_APP_DL_BLOCK0 [ROTARY_APP_START_FIFO_SIZE_INWORDS] =
        {	
          SCISSOR_SIZE(512,512),
	  SCISSOR_XY(0,0),
 	  /* Draw all the background bitmaps - balls with various resolutions */
	  BEGIN(BITMAPS),
	  COLOR_RGB(255 ,255 ,255), // brown outside circle
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
#ifdef DISPLAY_RESOLUTION_WQVGA
  	  POINT_SIZE(((DispWidth/4) - 19) * 16),		
#endif
#ifdef DISPLAY_RESOLUTION_QVGA
  	  POINT_SIZE(((DispWidth/4) + 5) * 16),		
#endif
	  BEGIN(FTPOINTS),
	  COLOR_RGB(235 ,117 ,0), // brown outside circle
	  VERTEX2F((DispWidth/2)*16 - 24,(DispHeight/2)*16 - 24),
	  COLOR_RGB(0 ,0 ,0), // brown outside circle
	  VERTEX2F((DispWidth/2)*16 + 24,(DispHeight/2)*16 + 24),
	  COLOR_RGB(128 ,64 ,0), // brown outside circle
	  VERTEX2F((DispWidth/2)*16,(DispHeight/2)*16),
#ifdef DISPLAY_RESOLUTION_WQVGA
	  POINT_SIZE(((DispWidth/4) - 19 - 50) * 16) ,
#endif
#ifdef DISPLAY_RESOLUTION_QVGA
	  POINT_SIZE(40*16) ,
#endif
	  COLOR_RGB(87 ,64 ,8), // inner circle - light brown color
	  VERTEX2F((DispWidth/2)*16 - 24,(DispHeight/2)*16 - 24),
	  COLOR_RGB(0 ,0 ,0), // inner circle - light brown color
	  VERTEX2F((DispWidth/2)*16 + 24,(DispHeight/2)*16+24),
	  COLOR_RGB(204 ,151 ,16), // inner circle - light brown color
	  VERTEX2F((DispWidth/2)*16,(DispHeight/2)*16),
        };
        
         PROGMEM prog_uint32_t ROTARY_APP_DL_BLOCK1 [ROTARY_APP_BLOCK1_SIZE_INWORDS] =   
        {     
		 CMD_DLSTART,                
		 CLEAR_COLOR_RGB(237 ,186 ,17) , // background color
		CLEAR(1,1,1),
                /* Draw two gradients, one at the top and another at the bottom. Height of former is 
                   80% of display height and later 20% of display height */
		SCISSOR_SIZE(512,218),
		SCISSOR_XY(0,0),
		//Gpu_CoCmd_Gradient(phost,0, 0, 0xEDBA11,0, DispHeight, 0x82660A);
                CMD_GRADIENT,
                0,
                0xEDBA11,
                (((uint32_t)DispHeight<<16)),
                0x82660A,

		SCISSOR_SIZE(512,512),
		SCISSOR_XY(0,218),
		//Gpu_CoCmd_Gradient(phost,0, 0, 0x2B2205,0, (DispHeight -(DispHeight*0.20))/*218*/, 0x82660A);                
                CMD_GRADIENT,
                0,
                0x2B2205,
              (((uint32_t)(DispWidth*0.8)<<16)),
              0x82660A,                
                };
                
                
                 PROGMEM prog_uint32_t ROTARY_APP_DL_BLOCK2 [ROTARY_APP_END_FIFO_SIZE_INWORDS] =   
                {     
		  //CMD_DLSTART,  
                  STENCIL_FUNC(GEQUAL,1,255),
		  COLOR_RGB(0 ,0 ,0),
		  BEGIN(BITMAPS),
#ifdef DISPLAY_RESOLUTION_WQVGA 
	          VERTEX2II(288,177,28,'0'),VERTEX2II(256,196,28,'9'),VERTEX2II(215,196,28,'8'),VERTEX2II(181,177,28,'7'),VERTEX2II(161,144,28,'6'),
                  VERTEX2II(161,105,28,'5'),VERTEX2II(181,70,28,'4'),VERTEX2II(215,50,28,'3'),VERTEX2II(256,50,28,'2'),VERTEX2II(288,70,28,'1'),
 #endif
#ifdef DISPLAY_RESOLUTION_QVGA 
                   VERTEX2II(191,151,28,'0'),VERTEX2II(164,164,28,'9'),VERTEX2II(135,161,28,'8'),VERTEX2II(111,144,28,'7'),VERTEX2II(98,117,28,'6'),
                  VERTEX2II(101,88,28,'5'),VERTEX2II(118,64,28,'4'),VERTEX2II(145,51,28,'3'),VERTEX2II(174,54,28,'2'),VERTEX2II(198,71,28,'1'),
#endif         
		  STENCIL_OP(KEEP,KEEP),
                  STENCIL_FUNC(ALWAYS,0,255),
                  COLOR_A(128),
		
#ifdef DISPLAY_RESOLUTION_WQVGA 
	          VERTEX2II(288,177,28,'0'),VERTEX2II(256,196,28,'9'),VERTEX2II(215,196,28,'8'),VERTEX2II(181,177,28,'7'),VERTEX2II(161,144,28,'6'),
                  VERTEX2II(161,105,28,'5'),VERTEX2II(181,70,28,'4'),VERTEX2II(215,50,28,'3'),VERTEX2II(256,50,28,'2'),VERTEX2II(288,70,28,'1'),
 #endif
#ifdef DISPLAY_RESOLUTION_QVGA
                   VERTEX2II(191,151,28,'0'),VERTEX2II(164,164,28,'9'),VERTEX2II(135,161,28,'8'),VERTEX2II(111,144,28,'7'),VERTEX2II(98,117,28,'6'),
                  VERTEX2II(101,88,28,'5'),VERTEX2II(118,64,28,'4'),VERTEX2II(145,51,28,'3'),VERTEX2II(174,54,28,'2'),VERTEX2II(198,71,28,'1'),
#endif      

		  COLOR_RGB(255,255,255),
		  COLOR_A(255),
		  BEGIN(BITMAPS),		

		  /* dial image*/
		  COLOR_RGB(255,255,255),
		  TAG_MASK(1),
		  TAG(100),

		  VERTEX2II(((DispWidth/2)-131),((DispHeight)-70),2,0),
		  TAG_MASK(0),	
                  /* fingerstop jpg */		
		  COLOR_RGB(255,255,255),
#ifdef DISPLAY_RESOLUTION_WQVGA
		  VERTEX2II(((DispWidth/2) + 56)/*296*/,((DispHeight/2) + 17)/*153*/,0,0), //exact location of picture	
 #endif
#ifdef DISPLAY_RESOLUTION_QVGA
                   VERTEX2II(((DispWidth/2) + 42)/*296*/,((DispHeight/2) + 10)/*153*/,0,0), //exact location of picture	
#endif
		  /*reject button jpg*/
		  COLOR_RGB(255,255,255),
		  TAG_MASK(1),
		  TAG(200),
		  VERTEX2II(((DispWidth/2)+80),((DispHeight)-70),1,0),
		  TAG_MASK(0),
		 
                 CMD_FGCOLOR,
                 0xCC9710,

		
               CMD_BUTTON,
               ((uint32_t)((uint32_t)5<<16)| (DispWidth/6)),               
#ifdef DISPLAY_RESOLUTION_WQVGA 
               (((uint32_t)((uint32_t)((DispHeight/2) - 111)<<16)|(DispWidth - 160))),   
 #endif
#ifdef DISPLAY_RESOLUTION_QVGA
               ((uint32_t)((uint32_t)((DispHeight/2) - 95)<<16)|(DispWidth - 110)),
#endif
               (((uint32_t)0<<16)|28),
               0,

		
                };
        
#endif


void Rotary_Dial_phone()
{


	uint8_t ReadTag = 0,PrevTag = 0,StoreTag,flag =0, PrintTag=0,Tagcheck=0,FontWid = 0, FontHt = 0,Penup=0;
	char8_t StringArray[31];	
	uint32_t temp =0,tagval,PrevTh = 0,adjusting = 0, CurrTh=0,VertexX, VertexY;	
	uint16_t OuterCircleRadius=0, InnerCircleRadius,CentreWhiteCircle,PrevTouch=0;
	uint16_t ScreenWidthMid,ScreenHeightMid,FontPLaceX[10],FontPLaceY[10],SmallNumCircleRad, i,j,k=0,l=0,m=0;
	uint16_t NoX,NoY,vX,MinTheta,MaxTheta,thresholdflag=0,ButHt, ButWid,FingerstopX,FingerstopY,th,theta;	
	uint32_t RotaryTag=0,RamFifoStartAddr,RamFifoendsize,RamAddrOffest;
	int16_t threshold_angle =320, StartTheta, EndTheta, IncTheta;
        int16_t NoTouch,Touch, NextTh=0;
	RotaryDial_Bmps_t BmpBalls[40];
	
	ScreenWidthMid = DispWidth/2;
	ScreenHeightMid = DispHeight/2; 
      	
        uint32_t time = 0;

#ifdef DISPLAY_RESOLUTION_WQVGA
	OuterCircleRadius = ScreenWidthMid/2 - 19;	
	InnerCircleRadius = OuterCircleRadius - 50;
	SmallNumCircleRad = InnerCircleRadius/2 - 8;
	FingerstopX = ScreenWidthMid + 56;
	FingerstopY = ScreenHeightMid + 17;
	NoX = DispWidth/6;//80; //(ScreenWidthMid)-(ScreenWidthMid/2);
	NoY = 5;//DispHeight - (DispHeight - 7);//7;
	ButHt = (DispHeight/2) - 111;
	ButWid = DispWidth - 160;
#endif

#ifdef DISPLAY_RESOLUTION_QVGA
	OuterCircleRadius = ScreenWidthMid/2 + 5;	
	InnerCircleRadius = 30;
	SmallNumCircleRad = 13;
	FingerstopX = ScreenWidthMid + 42;
	FingerstopY = ScreenHeightMid +10;
	NoX = DispWidth/6;//80; //(ScreenWidthMid)-(ScreenWidthMid/2);
	NoY = DispHeight - (DispHeight - 7);//7;
	ButHt = (DispHeight/2) - 95;
	ButWid = DispWidth - 110;
#endif
    MinTheta = 60;
	MaxTheta = 90;
	CentreWhiteCircle = InnerCircleRadius + ((OuterCircleRadius - InnerCircleRadius)/2);	
	VertexX = 0;
	VertexY = 0;	
	StartTheta = 45;
	EndTheta = 270+45;
	IncTheta = 30;
	NoTouch = -32768;

	vX = ScreenWidthMid;
	StringArray[0] = '\0';
	tagval = 10;


	Gpu_Hal_WrCmd32(phost, CMD_MEMSET);
	Gpu_Hal_WrCmd32(phost, 0L);//starting address of memset
	Gpu_Hal_WrCmd32(phost, 55L);//value of memset
	Gpu_Hal_WrCmd32(phost,256*1024);
        i =0;



	/* set the vol to max */
	Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xFF);

	/* Calculation of balls offsets and rate */
	for(i=0;i<40;i++)
	{
		BmpBalls[i].xOffset = random(DispWidth);
		BmpBalls[i].yOffset = random(DispHeight);

		BmpBalls[i].xDiff = random(8) - 4;
		BmpBalls[i].yDiff = random(4) - 2;
	}

	/* Construction of starting screen shot, assign all the bitmap handles here */
	App_WrCoCmd_Buffer(phost, CMD_DLSTART);
	App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));
	App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	
	/* track circles outside the while loop */
	for(theta=StartTheta;theta<=EndTheta ;theta+=IncTheta)			
	{	
		th = theta;		
		if(theta == StartTheta + IncTheta) tagval = 9;
		th = 2*da(th,-90);				
		Gpu_CoCmd_Track(phost,ScreenWidthMid, ScreenHeightMid, DIAL, DIAL, tagval);		
		tagval--;
	}
	
	/* Download the raw data into intended locations */
	{
		RamAddrOffest = RAM_G;
		App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));

                RamAddrOffest = Load_RawDataFromfile(FingerStop,FingerStop_SIZE,RamAddrOffest,0,ARGB4,56*2,56,38,0);//finger stopper
		RamAddrOffest = Load_RawDataFromfile(Call,CallIcon,RamAddrOffest,1,ARGB4,56*2,56,56,1);//reject button
		RamAddrOffest = Load_RawDataFromfile(CallCancel,CallCancelIcon,RamAddrOffest,2,ARGB4,56*2,56,56,2);//call button
		RamAddrOffest = Load_RawDataFromfile(YellowBall20x20,YellowBall_20x20,RamAddrOffest,3,ARGB4,20*2,20,20,3);//ball 20x20
		RamAddrOffest = Load_RawDataFromfile(YellowBall30x30,YellowBall_30x30,RamAddrOffest,4,ARGB4,30*2,30,30,4);//ball 30x30
		RamAddrOffest = Load_RawDataFromfile(YellowBall40x40,YellowBall_40x40,RamAddrOffest,5,ARGB4,40*2,40,40,5);//ball 40x40
		RamAddrOffest = Load_RawDataFromfile(YellowBall60x60,YellowBall_60x60,RamAddrOffest,6,ARGB4,60*2,60,60,6);//ball 60x60

		App_WrCoCmd_Buffer(phost, END());
	}


	App_WrCoCmd_Buffer(phost,DISPLAY());
	Gpu_CoCmd_Swap(phost);

	/* Download the commands into fifo */
	App_Flush_Co_Buffer(phost);

	/* Wait till coprocessor completes the operation */
	Gpu_Hal_WaitCmdfifo_empty(phost);

        /* Performance optimization - Construct static array of graphics instructions once and download into GRAM and run time use cmd_append() */
        {
          RamFifoStartAddr = RamAddrOffest;

          Gpu_Hal_WrMemFromFlash(phost,RamFifoStartAddr,(prog_uchar8_t*)ROTARY_APP_DL_BLOCK0,ROTARY_APP_START_FIFO_SIZE);
             
          RamAddrOffest += ROTARY_APP_START_FIFO_SIZE;  
                
        }
        

	do{
                //Serial.println(millis()-time,DEC);   
                //time = millis();   

                /* Generate random bitmap offsets into the RGAM address directly and followed by Coprocessor append command */


                Gpu_Hal_StartTransfer(phost,GPU_WRITE,RamFifoStartAddr + ROTARY_APP_START_FIFO_RANDOMBITMAP_SIZE);
		for(i = 0;i<40;i++)
		{
			Gpu_Hal_Transfer32(phost,VERTEX2II(BmpBalls[i].xOffset,BmpBalls[i].yOffset,3+(i%4),0));
			BmpBalls[i].xOffset += BmpBalls[i].xDiff;
			BmpBalls[i].yOffset += BmpBalls[i].yDiff;

			if(BmpBalls[i].xOffset < -80)
			{
				BmpBalls[i].xOffset = DispWidth + 80;
			}
			else if(BmpBalls[i].xOffset > DispWidth + 80)
			{
				BmpBalls[i].xOffset = -80;
			}

			if(BmpBalls[i].yOffset < -80)
			{
				BmpBalls[i].yOffset = DispHeight + 80;
			}
			else if(BmpBalls[i].yOffset > DispHeight + 80)
			{
				BmpBalls[i].yOffset = -80;
			}
		}
                Gpu_Hal_EndTransfer(phost);

		BmpBalls[random(40)].xDiff = random(8) - 4;
		BmpBalls[random(40)].yDiff = random(4) - 2;

                /* use coprocessor append command to download the DL commands into DL memory from graphics ram */

                /* Array optimization */





                Gpu_Hal_WrCmdBufFromFlash(phost,(prog_uchar8_t*)ROTARY_APP_DL_BLOCK1,ROTARY_APP_BLOCK1_SIZE);
                Gpu_CoCmd_Append(phost,RamFifoStartAddr,ROTARY_APP_START_FIFO_SIZE);
              

		i = 0;//font inside the white circles - start at 0
		tagval = 10;// tag for the white circles - decrementing tagvalue inside loop


		Touch = Gpu_Hal_Rd16(phost, REG_TOUCH_SCREEN_XY+2);
		Tagcheck = Gpu_Hal_Rd8(phost, REG_TOUCH_TAG);

                RotaryTag = Gpu_Hal_Rd32(phost,REG_TRACKER);
		ReadTag = RotaryTag & 0xff;
                /* Anticlockwise movement */
		if((PrevTouch&0x8000) && (0 == PrevTag))
		{
			BaseTrackValInit = (int16_t)((RotaryTag >> 16) & 0xffff);
		}
		else
		{
			BaseTrackValInit = 0;
		}
                /* verify for the track value */
                if((Touch != NoTouch))
		{
			if((ReadTag >= 1) && (ReadTag <= 10))
			{
				CurrTh = (int16_t)((RotaryTag >> 16) & 0xffff);
                                /* BaseTrackVal is the rotary track value of the particular white circle that is moving */
				if(adjusting!=0)
				{
					flag = 0;
					BaseTrackVal += (int16_t)(CurrTh - PrevTh);
					BaseTrackValSign += (int16_t)(CurrTh - PrevTh);
					/* handling of loopback */
					BaseTrackVal &= 0xffff;	
                                        /* NextTh value is the track value in degrees */					
					NextTh = (uint16_t)(BaseTrackValSign / 182);	
                                        //NextTh = abs(NextTh);

				
				} 
				PrevTh = CurrTh;
				adjusting = (RotaryTag & 0xff);
				PrintTag = ReadTag;
			
			}
		}
                else if(Touch == NoTouch)
                {
			BaseTrackVal=0;		
			flag = 1;
                        Penup = 1;
			if(adjusting != 0)
			{        
                                /* If white circles have moved beyond their initial value, Nextth value is incremented */
				if(NextTh > 0 )
				{
                                        /* After the white circle has moved to the desired location, and when penup has been done,
                                        the white crcles moved back to its original positon of 0 degrees. */
                                        /* NextTh moves back in degrees of twenty in steps. Also sound has to be played when moving back. */
					NextTh-=20;	
					Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xff);
					Gpu_Hal_Wr16(phost, REG_SOUND,0x51);
					Gpu_Hal_Wr8(phost, REG_PLAY,1);
					MinTheta = 60;
					MaxTheta = 90;
					
				}
				else
				{ 
    					/* When the white circles have moved back to its original position, the tag value of the particular white circle 
                                               (PrintTag) is stored in the StringArray. StringArray stores upto 17 values. The thresholdflag is also checked 
                                               before entering this loop */
					NextTh = 0;
					if((PrintTag >= 1) && (PrintTag <= 10) && (thresholdflag ==1))
					{
						if (PrintTag == 10)
                                                {
                                                  PrintTag = 0;
                                                }
                                                
						StringArray[k] = PrintTag + '0';

						if (k >= 17)
						{
							k = 0;
							StringArray[k] = PrintTag + '0';
							
						}
                                                k++;
						thresholdflag=0;
					}
                            
					StringArray[k] = '\0';					
					adjusting = 0;
					
				}					
			}
                        /* When movement has been finished, the RotaryTag and Printtag are cleared*/
			else if(adjusting == 0)
			{
				RotaryTag = '\0';
				PrintTag = '\0';			
				
			}
			BaseTrackValSign = 0;
		}

                /* Sound play wrt user touch */
                /* When white circles are touched for the first time after penup - play sound */
                if((PrevTag == 0)&& ((ReadTag !=0)&&(ReadTag !=255)) && ( Penup = 1))
                {
		      Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xFF);
		      Gpu_Hal_Wr16(phost, REG_SOUND,0x51);
		      Gpu_Hal_Wr8(phost, REG_PLAY,1);
                      Penup = 0;
                }
  
		/* dial button (tag 100) & penup has been done - play dtmf sound */
		if((Tagcheck == 100)  && (flag == 1))
		{
			if(k>0)
			{				
				for(i=0;i<k;i++)
				{
					/* Play the dtmf sound */
					Gpu_Hal_Wr16(phost, REG_SOUND,StringArray[i]);
					Gpu_Hal_Wr8(phost, REG_PLAY,1);
                                        Gpu_Hal_Sleep(80);
					Gpu_Hal_Wr16(phost, REG_SOUND,0);
					Gpu_Hal_Wr8(phost, REG_PLAY,1);
					Gpu_Hal_Sleep(80);
				}
			}

		}

		/* reject button (tag -200)& penup has been done. clear the string */
		if((Tagcheck == 200) && (flag ==1) )
		{
		        StringArray[0] = '\0';
			k=0;
		}                  

                Gpu_Hal_CheckCmdBuffer(phost,ROTARY_APP_WHITECIRCELE_FIFO_SIZE);
                Gpu_Hal_StartTransfer(phost,GPU_WRITE,RAM_CMD + phost->cmd_fifo_wp);
		Gpu_Hal_Transfer32(phost,BEGIN(FTPOINTS));
		Gpu_Hal_Transfer32(phost,STENCIL_OP(INCR,INCR));//pass or fail increment	
		Gpu_Hal_Transfer32(phost,POINT_SIZE(SmallNumCircleRad * 16) );
		Gpu_Hal_Transfer32(phost,COLOR_A(255));
		Gpu_Hal_Transfer32(phost,COLOR_RGB(255 ,255 ,255));	

		/* Placement of number circles which will move around the rotary */		
                Gpu_Hal_Transfer32(phost,TAG_MASK(1));
		for(theta=StartTheta;theta<=EndTheta ;theta+=IncTheta)			
		{	
			
			th = theta;		
			if(theta == (StartTheta + IncTheta)) 
                        {
                          tagval = 9;	
                        }
			
			/* limit threshold level near finger stop*/
			/* for value 0 - tag is 10*/
			if((NextTh > threshold_angle) && (ReadTag == 10))
			{
				NextTh = threshold_angle;
				thresholdflag =1;
			}
			
			/* for value 1 to 9 - threshold*/
			for(m = 1,l=9; m<=9, l>=1;m++,l--)
			{
				if((NextTh > threshold_angle - (m*IncTheta)) && (ReadTag == l))
				{
				    NextTh = threshold_angle - (m*IncTheta);
		                    thresholdflag =1;
				}
			}
			/* To stop anti clockwise movement of the dial */			
			if((BaseTrackValInit + BaseTrackValSign) < BaseTrackValInit)
			{
				NextTh = 0;
			}
			th = th + NextTh;		
			/* calculation of th value in pixels with input in degrees */
			th = 2*da(th,-90);	

			/* calculate x and  y for placing circles according to degree */
			polarxy(CentreWhiteCircle,th,&VertexX ,&VertexY ,ScreenWidthMid,ScreenHeightMid);	
			
		
			Gpu_Hal_Transfer32(phost,TAG(tagval));
			Gpu_Hal_Transfer32(phost,COLOR_RGB(200 ,99 ,0)); // for 3d effect adding circles of darker color
			Gpu_Hal_Transfer32(phost,VERTEX2F(VertexX-24,VertexY-24));
			Gpu_Hal_Transfer32(phost,COLOR_RGB(0 ,0 ,0)); 
			Gpu_Hal_Transfer32(phost,VERTEX2F(VertexX+24,VertexY+24));// for 3d effect adding circles of darker color
			Gpu_Hal_Transfer32(phost,COLOR_RGB(255 ,255 ,255));	
			Gpu_Hal_Transfer32(phost,VERTEX2F(VertexX,VertexY));// the base number circle

			tagval--;	
			
		}
                Gpu_Hal_Transfer32(phost,TAG_MASK(0));			
                Gpu_Hal_EndTransfer(phost);
                Gpu_Hal_Updatecmdfifo(phost,ROTARY_APP_WHITECIRCELE_FIFO_SIZE);
                
                /* When NextTh value moves between 30 degrees, play sound */
            	if((NextTh > MaxTheta) && (Touch != NoTouch))
		{
			Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xFF);
			Gpu_Hal_Wr16(phost, REG_SOUND,0x51);
			Gpu_Hal_Wr8(phost, REG_PLAY,1);
			MinTheta = MinTheta + IncTheta;
			MaxTheta = MaxTheta + IncTheta;		
		}
		else if(((NextTh < MinTheta) && (NextTh >(MinTheta-30))) && (NextTh!=0)&&(Touch != NoTouch))
		{
			Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xFF);
			Gpu_Hal_Wr16(phost, REG_SOUND,0x51);
			Gpu_Hal_Wr8(phost, REG_PLAY,1);
			MinTheta = MinTheta - IncTheta;
			MaxTheta = MaxTheta - IncTheta;		
		}
        

		/* Placement of NUmbers inside the white circle - using stencil effect*/
                RamFifoendsize = 0;
                
                Gpu_Hal_WrCmdBufFromFlash(phost,(prog_uchar8_t*)ROTARY_APP_DL_BLOCK2,ROTARY_APP_END_FIFO_SIZE);
                

		/* display numbers in the centre of the point while dialing*/
		if((PrintTag >0 ) && (PrintTag <= 10) && (Touch != NoTouch))
		{
      		        Gpu_Hal_StartTransfer(phost,GPU_WRITE,RAM_CMD + phost->cmd_fifo_wp);
                        /* The tag value of zero font is 10. So Set PrintTag is set to zero to print it correctly */
			if (PrintTag == 10)PrintTag = 0;
			Gpu_Hal_Transfer32(phost,COLOR_RGB(0,0,0));
			
                        Gpu_Hal_Transfer32(phost,VERTEX2II((ScreenWidthMid - 16)/*225*/,(ScreenHeightMid - 27)/*110*/,31,PrintTag + '0'));

			Gpu_Hal_Transfer32(phost,COLOR_RGB(0,0,0));
			
                        Gpu_Hal_Transfer32(phost,VERTEX2II((ScreenWidthMid - 14)/*225*/,(ScreenHeightMid - 25)/*110*/,31,PrintTag + '0'));
			Gpu_Hal_Transfer32(phost,COLOR_RGB(255,255,255));
			
                        Gpu_Hal_Transfer32(phost,VERTEX2II((ScreenWidthMid - 15)/*225*/,(ScreenHeightMid - 26)/*110*/,31,PrintTag + '0'));
                        /* Restore the PrintTag to ten after printing has been done */
			if (PrintTag == 0)PrintTag = 10;
                        RamFifoendsize = 6*4;
                        Gpu_Hal_EndTransfer(phost);
        
                        Gpu_Hal_Updatecmdfifo(phost,RamFifoendsize);                    
		}

                App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));

		Gpu_CoCmd_Text(phost,(NoX + ButWid/2),(NoY + ButHt/2),28,OPT_CENTER,StringArray);


		App_WrCoCmd_Buffer(phost,DISPLAY());                
                
		Gpu_CoCmd_Swap(phost);

		/* Download the commands into fifo */
		App_Flush_Co_Buffer(phost);

		/* Wait till coprocessor completes the operation */
		Gpu_Hal_WaitCmdfifo_empty(phost);

                /* update all the prev with current */
		PrevTag = ReadTag;
		PrevTouch = Touch;

		}while(1);
			

}


/* Startup Screen Information*/ 

#if defined(ARDUINO_PLATFORM)
prog_char8_t * const info[] =
#else
char *info[] =
#endif 
{  "EVE Rotary Dial Application",
    "APP to demonstrate rotary dial",
    "using Points, Track",
    "& Stencil"
}; 

/* Boot up for FT800 followed by graphics primitive sample cases */
/* Initial boot up DL - make the back ground green color */

#if defined MSVC_PLATFORM | defined FT9XX_PLATFORM
/* Main entry point */
int32_t main(int32_t argc,char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM) || defined(MSVC_FT800EMU)
void setup()
#endif
{

    phost = &host;
    /* Init HW Hal */
    App_Common_Init(phost);
    /* Screen Calibration*/
    App_Calibrate_Screen(phost);
    /* Main application */
    Rotary_Dial_phone();   

    /* Close all the opened handles */
    App_Common_Close(phost);	

#ifdef MSVC_PLATFORM
	return 0;
#endif
}

void loop()
{
}



/* Nothing beyond this */
